"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JiraFigmaMcpServer = void 0;
exports.createJiraFigmaMcpServer = createJiraFigmaMcpServer;
const mcp_js_1 = require("@modelcontextprotocol/sdk/server/mcp.js");
const zod_1 = require("zod");
const sse_js_1 = require("@modelcontextprotocol/sdk/server/sse.js");
const jira_service_1 = require("../services/jira/jira-service");
const figma_service_1 = require("../services/figma/figma-service");
const url_parser_1 = require("../utils/url-parser");
const config_1 = require("../config");
/**
 * Helper function to standardize response formatting
 * @param data The data to format in the response
 * @returns Formatted response object
 */
function formatResponse(data) {
    return {
        content: [{ type: "text", text: JSON.stringify(data, null, 2) }],
    };
}
/**
 * Format large responses that might exceed string size limits
 * @param data Data with potentially large arrays to format
 * @param arrayProp Property name containing large array data
 * @returns Formatted response object
 */
function formatLargeResponse(data, arrayProp) {
    // Create a copy without the large array
    const { [arrayProp]: largeArray, ...metadata } = data;
    // Handle case where the large array doesn't exist
    if (!largeArray) {
        return formatResponse(data);
    }
    // Format regular metadata
    const metadataJson = JSON.stringify(metadata, null, 2);
    // Format large array items individually to avoid string size limits
    const arrayJson = `[${largeArray.map((item) => JSON.stringify(item, null, 2)).join(",")}]`;
    // Combine into a valid JSON string
    const resultJson = `{ "metadata": ${metadataJson}, "${arrayProp}": ${arrayJson} }`;
    return {
        content: [{ type: "text", text: resultJson }],
    };
}
/**
 * MCP Server that provides tools for interacting with Jira and Figma
 */
class JiraFigmaMcpServer {
    constructor() {
        this.sseTransport = null;
        // Standard regex for extracting Figma URLs
        this.figmaUrlRegex = /https:\/\/(?:www\.)?figma\.com\/(?:file|design|proto)\/([a-zA-Z0-9]+)(?:\/[^?#]+)?(?:\?.*node-id=([^&]+))?/g;
        this.jiraService = jira_service_1.jiraService;
        this.figmaService = new figma_service_1.FigmaService(config_1.config.figma.accessToken);
        this.server = new mcp_js_1.McpServer({
            name: "Jira-Figma MCP Server",
            version: "1.0.0",
        });
        this.registerTools();
    }
    /**
     * Register all MCP tools
     */
    registerTools() {
        this.registerJiraTools();
        this.registerFigmaTools();
        this.registerLinkingTools();
    }
    /**
     * Register Jira-specific tools
     */
    registerJiraTools() {
        this.server.tool("get_ticket", "Get detailed information about a specific Jira ticket", {
            ticket_id: zod_1.z.string().describe("The Jira ticket ID (e.g., PROJECT-123)")
        }, async ({ ticket_id }) => {
            const ticket = await this.jiraService.getTicket(ticket_id);
            return formatResponse({ ticket });
        });
    }
    /**
     * Register Figma-specific tools
     */
    registerFigmaTools() {
        this.server.tool("get_figma_data", "When the nodeId cannot be obtained, obtain the layout information about the entire Figma file", {
            fileKey: zod_1.z.string().describe("The key of the Figma file to fetch, often found in a provided URL like figma.com/(file|design)/<fileKey>/..."),
            nodeId: zod_1.z.string().optional().describe("The ID of the node to fetch, often found as URL parameter node-id=<nodeId>, always use if provided"),
            depth: zod_1.z.number().optional().describe("How many levels deep to traverse the node tree, only use if explicitly requested by the user")
        }, async ({ fileKey, nodeId, depth }) => {
            try {
                let design;
                if (nodeId) {
                    design = await this.figmaService.getNode(fileKey, nodeId, depth);
                }
                else {
                    design = await this.figmaService.getFile(fileKey, depth);
                }
                return formatLargeResponse(design, "nodes");
            }
            catch (error) {
                console.error(`Error fetching file ${fileKey}:`, error);
                return formatResponse({ error: `Error fetching file: ${error}` });
            }
        });
    }
    /**
     * Register tools that link Jira and Figma data
     */
    registerLinkingTools() {
        this.server.tool("get_designs_for_ticket", "Get Figma design information linked to a Jira ticket", {
            ticket_id: zod_1.z.string().describe("The Jira ticket ID (e.g., PROJECT-123)"),
            depth: zod_1.z.number().optional().describe("Depth to traverse the Figma node tree")
        }, async ({ ticket_id, depth }) => {
            try {
                // Get ticket information
                const ticket = await this.jiraService.getTicket(ticket_id);
                // Extract Figma URLs from the ticket
                const figmaUrls = this.extractFigmaUrlsFromTicket(ticket);
                if (figmaUrls.length === 0) {
                    return formatResponse({
                        ticket_id,
                        ticket,
                        designs: [],
                        message: 'No Figma designs found in this ticket'
                    });
                }
                // Fetch design data for each URL
                const designs = await Promise.all(figmaUrls.map(async (urlInfo) => {
                    try {
                        let design;
                        if (urlInfo.nodeId) {
                            design = await this.figmaService.getNode(urlInfo.fileKey, urlInfo.nodeId, depth);
                        }
                        else {
                            design = await this.figmaService.getFile(urlInfo.fileKey, depth);
                        }
                        return {
                            url: urlInfo.url,
                            fileKey: urlInfo.fileKey,
                            nodeId: urlInfo.nodeId,
                            design
                        };
                    }
                    catch (error) {
                        console.error(`Failed to fetch design for fileKey ${urlInfo.fileKey}:`, error);
                        return {
                            url: urlInfo.url,
                            fileKey: urlInfo.fileKey,
                            nodeId: urlInfo.nodeId,
                            error: `Failed to fetch design: ${error}`
                        };
                    }
                }));
                return formatResponse({
                    ticket_id,
                    ticket_summary: ticket.summary || '',
                    ticket,
                    designs
                });
            }
            catch (error) {
                console.error(`Error processing ticket ${ticket_id}:`, error);
                return formatResponse({ error: `Error processing ticket: ${error}` });
            }
        });
    }
    /**
     * Helper function to extract text from rich text content
     * @param content Rich text content object
     * @returns Extracted plain text
     */
    extractText(content) {
        if (!content)
            return '';
        if (typeof content === 'string')
            return content;
        let text = '';
        if (content.content && Array.isArray(content.content)) {
            for (const item of content.content) {
                if (item.text) {
                    text += item.text;
                }
                else if (item.content) {
                    text += this.extractText(item);
                }
            }
        }
        return text;
    }
    /**
     * Clean a URL by handling escape sequences
     * @param url URL to clean
     * @returns Cleaned URL
     */
    cleanUrl(url) {
        return url.replace(/\\"/g, '"').replace(/\\u0026/g, '&');
    }
    /**
     * Parse a Figma URL to extract fileKey and nodeId
     * @param url Figma URL to parse
     * @returns Parsed URL info or null if invalid
     */
    parseFigmaUrl(url) {
        // Use the existing utility function if possible
        const parsedUrl = (0, url_parser_1.parseFigmaUrl)(url);
        if (parsedUrl && parsedUrl.fileKey) {
            return {
                url,
                fileKey: parsedUrl.fileKey,
                nodeId: parsedUrl.nodeId
            };
        }
        // Fallback to regex parsing if utility function fails
        this.figmaUrlRegex.lastIndex = 0; // Reset regex state
        const match = this.figmaUrlRegex.exec(url);
        if (match) {
            return {
                url,
                fileKey: match[1],
                nodeId: match[2]
            };
        }
        return null;
    }
    /**
     * Extract all Figma URLs from a Jira ticket
     * @param ticket The Jira API response
     * @returns Array of parsed Figma URLs
     */
    extractFigmaUrlsFromTicket(ticket) {
        const rawTicketData = JSON.stringify(ticket);
        const results = [];
        // 1. First try to extract from href attributes (most common in Jira)
        const hrefMatches = rawTicketData.match(/"href":\s*"(https:\/\/(?:www\.)?figma\.com\/(?:file|design|proto)\/[^"]+)"/g) || [];
        for (const hrefMatch of hrefMatches) {
            const urlMatch = hrefMatch.match(/"href":\s*"([^"]+)"/);
            if (urlMatch && urlMatch[1]) {
                const cleanedUrl = this.cleanUrl(urlMatch[1]);
                const urlInfo = this.parseFigmaUrl(cleanedUrl);
                if (urlInfo) {
                    results.push(urlInfo);
                }
            }
        }
        // 2. Look for embedCard URLs
        if (ticket.fields?.description &&
            typeof ticket.fields.description === 'object' &&
            ticket.fields.description.content) {
            const content = ticket.fields.description.content;
            if (Array.isArray(content)) {
                for (const item of content) {
                    // Check for embedCard type with URL attribute
                    if (item.type === 'embedCard' && item.attrs?.url && item.attrs.url.includes('figma.com')) {
                        const urlInfo = this.parseFigmaUrl(item.attrs.url);
                        if (urlInfo) {
                            results.push(urlInfo);
                        }
                    }
                }
            }
        }
        // 3. Extract from direct text mentions in description and comments
        const figmaUrlMatches = rawTicketData.match(/https:\/\/(?:www\.)?figma\.com\/(?:file|design|proto)\/[a-zA-Z0-9]+[^"\s\\]*/g) || [];
        for (const url of figmaUrlMatches) {
            const cleanedUrl = this.cleanUrl(url);
            const urlInfo = this.parseFigmaUrl(cleanedUrl);
            if (urlInfo) {
                results.push(urlInfo);
            }
        }
        // 4. Check for a dedicated Figma field if configured
        if (ticket.fields &&
            config_1.config.jira.figmaFieldName in ticket.fields &&
            ticket.fields[config_1.config.jira.figmaFieldName]) {
            const figmaField = ticket.fields[config_1.config.jira.figmaFieldName];
            let figmaValue = '';
            // Handle different field types
            if (typeof figmaField === 'string') {
                figmaValue = figmaField;
            }
            else if (typeof figmaField === 'object' && figmaField.value) {
                figmaValue = figmaField.value;
            }
            else if (Array.isArray(figmaField) && figmaField.length > 0) {
                const firstValue = figmaField[0];
                if (typeof firstValue === 'string') {
                    figmaValue = firstValue;
                }
                else if (typeof firstValue === 'object' && firstValue.value) {
                    figmaValue = firstValue.value;
                }
            }
            if (figmaValue && figmaValue.includes('figma.com')) {
                const urlInfo = this.parseFigmaUrl(figmaValue);
                if (urlInfo) {
                    results.push(urlInfo);
                }
            }
        }
        // Remove duplicates by URL
        const uniqueResults = [];
        const seenUrls = new Set();
        for (const result of results) {
            const normalizedUrl = result.url.split('?')[0]; // Remove query params for deduplication
            if (!seenUrls.has(normalizedUrl)) {
                seenUrls.add(normalizedUrl);
                uniqueResults.push(result);
            }
        }
        return uniqueResults;
    }
    /**
     * Connect to a transport mechanism
     */
    async connect(transport) {
        await this.server.connect(transport);
    }
    /**
     * Add HTTP transport for SSE connections
     * @param app Express app to attach to
     * @param ssePath SSE endpoint path
     * @param messagePath Message endpoint path
     */
    addHttpTransport(app, ssePath = '/mcp/sse', messagePath = '/mcp/messages') {
        // SSE endpoint for MCP communication
        app.get(ssePath, async (req, res) => {
            // Set headers for SSE
            res.setHeader('Content-Type', 'text/event-stream');
            res.setHeader('Cache-Control', 'no-cache');
            res.setHeader('Connection', 'keep-alive');
            this.sseTransport = new sse_js_1.SSEServerTransport(messagePath, res);
            await this.connect(this.sseTransport);
        });
        // Message endpoint for MCP communication
        app.post(messagePath, async (req, res) => {
            if (!this.sseTransport) {
                res.status(400).json({ error: "No SSE connection established" });
                return;
            }
            await this.sseTransport.handlePostMessage(req, res);
        });
    }
    /**
     * Add stdio transport for terminal/CLI use
     */
    addStdioTransport() {
        // For stdio transport, we need to use a custom transport
        const stdinTransport = {
            send: async (msg) => {
                process.stdout.write(msg + '\n');
            },
            receive: async (handler) => {
                process.stdin.on('data', async (data) => {
                    const message = data.toString().trim();
                    if (message) {
                        await handler(message);
                    }
                });
            }
        };
        this.connect(stdinTransport);
    }
    /**
     * Start listening on the configured transports
     */
    async start() {
        console.log('Jira-Figma MCP Server initialized');
    }
}
exports.JiraFigmaMcpServer = JiraFigmaMcpServer;
// Export a function to create a new server instance
function createJiraFigmaMcpServer() {
    return new JiraFigmaMcpServer();
}
//# sourceMappingURL=jira-figma-mcp-server.js.map