"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.jiraService = exports.JiraService = void 0;
const jira_client_1 = __importDefault(require("jira-client"));
const config_1 = require("../../config");
const errors_1 = require("../../utils/errors");
const cache_1 = require("../../utils/cache");
/**
 * Service for interacting with Jira API
 */
class JiraService {
    constructor() {
        // Initialize Jira client with configuration
        this.client = new jira_client_1.default({
            protocol: 'https',
            host: config_1.config.jira.host.replace(/^https?:\/\//, ''), // Remove protocol if present
            username: config_1.config.jira.username,
            password: config_1.config.jira.apiToken,
            apiVersion: '3',
            strictSSL: true
        });
        this.figmaFieldName = config_1.config.jira.figmaFieldName;
    }
    /**
     * Retrieves a list of Jira tickets based on project and status
     * @param project The Jira project key
     * @param status The ticket status to filter by (optional)
     * @returns Array of ticket objects with figma links
     */
    async listTickets(project, status) {
        try {
            // Build JQL query
            let jql = `project = "${project}"`;
            if (status) {
                jql += ` AND status = "${status}"`;
            }
            // Use cache to avoid redundant API calls
            const cacheKey = `jira_tickets_${project}_${status || 'all'}`;
            return (0, cache_1.getOrCreateCache)(cacheKey, async () => {
                // Make API request
                const response = await this.client.searchJira(jql, {
                    maxResults: 100,
                    fields: ['summary', 'description', 'status', this.figmaFieldName]
                });
                // Transform response to our ticket model
                return this.transformTickets(response.issues);
            });
        }
        catch (error) {
            if (error instanceof Error) {
                throw new errors_1.JiraError(`Failed to fetch Jira tickets: ${error.message}`, 500);
            }
            throw error;
        }
    }
    /**
     * Retrieves a specific Jira ticket by ID
     * @param ticketId The Jira ticket ID or key
     * @returns Ticket object with figma link
     */
    async getTicket(ticketId) {
        try {
            // Use cache to avoid redundant API calls
            const cacheKey = `jira_ticket_${ticketId}`;
            return (0, cache_1.getOrCreateCache)(cacheKey, async () => {
                // Make API request
                const issue = await this.client.findIssue(ticketId);
                // Transform response to our ticket model
                return this.transformTicket(issue);
            });
        }
        catch (error) {
            if (error instanceof Error) {
                throw new errors_1.JiraError(`Failed to fetch Jira ticket: ${error.message}`, 500);
            }
            throw error;
        }
    }
    /**
     * Transforms Jira API response to our ticket model
     * @param issues Array of Jira issues from API
     * @returns Array of transformed ticket objects
     */
    transformTickets(issues) {
        return issues.map(issue => this.transformTicket(issue));
    }
    /**
     * Transforms a single Jira issue to our ticket model
     * @param issue Jira issue from API
     * @returns Transformed ticket object
     */
    transformTicket(issue) {
        // Extract the Figma link from the custom field
        let figmaLink = '';
        // Get the value from the custom field
        const figmaFieldValue = issue.fields[this.figmaFieldName];
        // Handle different types of values the field might contain
        if (figmaFieldValue) {
            // If it's an object with a 'value' property (select/option fields)
            if (typeof figmaFieldValue === 'object' && figmaFieldValue.value) {
                figmaLink = figmaFieldValue.value;
            }
            // If it's a string (text fields)
            else if (typeof figmaFieldValue === 'string') {
                figmaLink = figmaFieldValue;
            }
            // If it's an array (multi-select fields)
            else if (Array.isArray(figmaFieldValue) && figmaFieldValue.length > 0) {
                // Take the first value
                const firstValue = figmaFieldValue[0];
                if (typeof firstValue === 'object' && firstValue.value) {
                    figmaLink = firstValue.value;
                }
                else if (typeof firstValue === 'string') {
                    figmaLink = firstValue;
                }
            }
        }
        // If figmaLink is not a URL but a text label, we'll still consider it valid
        if (figmaLink && !figmaLink.includes('figma.com')) {
            console.log(`Ticket ${issue.key} has a non-URL Figma reference: "${figmaLink}"`);
        }
        else if (!figmaLink) {
            console.warn(`Ticket ${issue.key} does not have a Figma link`);
        }
        return {
            id: issue.id,
            key: issue.key,
            summary: issue.fields.summary,
            description: issue.fields.description,
            status: issue.fields.status.name,
            figmaLink
        };
    }
}
exports.JiraService = JiraService;
// Export a singleton instance
exports.jiraService = new JiraService();
//# sourceMappingURL=jira-service.js.map