"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseFigmaResponse = parseFigmaResponse;
const layout_1 = require("../utils/transformers/layout");
const identity_1 = require("../utils/identity");
const common_1 = require("../utils/common");
const strokes_1 = require("../utils/transformers/strokes");
const effects_1 = require("../utils/transformers/effects");
// ---------------------- PARSING ----------------------
function parseFigmaResponse(data) {
    const { name, lastModified, thumbnailUrl } = data;
    let nodes;
    if ("document" in data) {
        nodes = Object.values(data.document.children);
    }
    else {
        nodes = Object.values(data.nodes).map((n) => n.document);
    }
    let globalVars = {
        styles: {},
    };
    const simplifiedNodes = nodes
        .filter(common_1.isVisible)
        .map((n) => parseNode(globalVars, n))
        .filter((child) => child !== null && child !== undefined);
    return {
        name,
        lastModified,
        thumbnailUrl: thumbnailUrl || "",
        nodes: simplifiedNodes,
        globalVars,
    };
}
// Helper function to find node by ID
const findNodeById = (id, nodes) => {
    for (const node of nodes) {
        if (node?.id === id) {
            return node;
        }
        if (node?.children && node.children.length > 0) {
            const foundInChildren = findNodeById(id, node.children);
            if (foundInChildren) {
                return foundInChildren;
            }
        }
    }
    return undefined;
};
/**
 * Find or create global variables
 * @param globalVars - Global variables object
 * @param value - Value to store
 * @param prefix - Variable ID prefix
 * @returns Variable ID
 */
function findOrCreateVar(globalVars, value, prefix) {
    // Check if the same value already exists
    const [existingVarId] = Object.entries(globalVars.styles).find(([_, existingValue]) => JSON.stringify(existingValue) === JSON.stringify(value)) ?? [];
    if (existingVarId) {
        return existingVarId;
    }
    // Create a new variable if it doesn't exist
    const varId = (0, common_1.generateVarId)(prefix);
    globalVars.styles[varId] = value;
    return varId;
}
function parseNode(globalVars, n, parent) {
    const { id, name, type } = n;
    const simplified = {
        id,
        name,
        type,
    };
    // text
    if ((0, identity_1.hasValue)("style", n) && Object.keys(n.style).length) {
        const style = n.style;
        const textStyle = {
            fontFamily: style.fontFamily,
            fontWeight: style.fontWeight,
            fontSize: style.fontSize,
            lineHeight: style.lineHeightPx && style.fontSize
                ? `${style.lineHeightPx / style.fontSize}em`
                : undefined,
            letterSpacing: style.letterSpacing && style.letterSpacing !== 0 && style.fontSize
                ? `${(style.letterSpacing / style.fontSize) * 100}%`
                : undefined,
            textCase: style.textCase,
            textAlignHorizontal: style.textAlignHorizontal,
            textAlignVertical: style.textAlignVertical,
        };
        simplified.textStyle = findOrCreateVar(globalVars, textStyle, "style");
    }
    // fills & strokes
    if ((0, identity_1.hasValue)("fills", n) && Array.isArray(n.fills) && n.fills.length) {
        // const fills = simplifyFills(n.fills.map(parsePaint));
        const fills = n.fills.map(common_1.parsePaint);
        simplified.fills = findOrCreateVar(globalVars, fills, "fill");
    }
    const strokes = (0, strokes_1.buildSimplifiedStrokes)(n);
    if (strokes.colors.length) {
        simplified.strokes = findOrCreateVar(globalVars, strokes, "stroke");
    }
    const effects = (0, effects_1.buildSimplifiedEffects)(n);
    if (Object.keys(effects).length) {
        simplified.effects = findOrCreateVar(globalVars, effects, "effect");
    }
    // Process layout
    const layout = (0, layout_1.buildSimplifiedLayout)(n, parent);
    if (Object.keys(layout).length > 1) {
        simplified.layout = findOrCreateVar(globalVars, layout, "layout");
    }
    // Keep other simple properties directly
    if ((0, identity_1.hasValue)("characters", n, identity_1.isTruthy)) {
        simplified.text = n.characters;
    }
    // border/corner
    // opacity
    if ((0, identity_1.hasValue)("opacity", n) && typeof n.opacity === "number" && n.opacity !== 1) {
        simplified.opacity = n.opacity;
    }
    if ((0, identity_1.hasValue)("cornerRadius", n) && typeof n.cornerRadius === "number") {
        simplified.borderRadius = `${n.cornerRadius}px`;
    }
    if ((0, identity_1.hasValue)("rectangleCornerRadii", n, identity_1.isRectangleCornerRadii)) {
        const radii = n.rectangleCornerRadii;
        simplified.borderRadius = `${radii[0]}px ${radii[1]}px ${radii[2]}px ${radii[3]}px`;
    }
    // Recursively process child nodes
    if ((0, identity_1.hasValue)("children", n) && n.children.length > 0) {
        let children = n.children
            .filter(common_1.isVisible)
            .map((child) => parseNode(globalVars, child, n))
            .filter((child) => child !== null && child !== undefined);
        if (children.length) {
            simplified.children = children;
        }
    }
    // Convert VECTOR to IMAGE
    if (type === "VECTOR") {
        simplified.type = "IMAGE-SVG";
    }
    return (0, common_1.removeEmptyKeys)(simplified);
}
//# sourceMappingURL=simplify-node-response.js.map