"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cache = void 0;
exports.getOrCreateCache = getOrCreateCache;
exports.clearCache = clearCache;
const node_cache_1 = __importDefault(require("node-cache"));
const config_1 = require("../config");
/**
 * Cache instance for storing frequently accessed data
 */
exports.cache = new node_cache_1.default({
    stdTTL: config_1.config.server.cacheTTL, // Time to live in seconds
    checkperiod: config_1.config.server.cacheTTL * 0.2, // Check for expired keys every 20% of TTL
    useClones: false // Don't clone objects (for performance)
});
/**
 * Gets an item from cache or executes the factory function to retrieve it
 * @param key The cache key
 * @param factory Function to execute if key is not in cache
 * @returns The cached or newly retrieved value
 */
async function getOrCreateCache(key, factory) {
    // Check if value exists in cache
    const cachedValue = exports.cache.get(key);
    if (cachedValue !== undefined) {
        return cachedValue;
    }
    // If not in cache, get from source
    const value = await factory();
    // Store in cache
    exports.cache.set(key, value);
    return value;
}
/**
 * Clears specific keys from the cache
 * @param keyPattern String or RegExp pattern to match keys to clear
 */
function clearCache(keyPattern) {
    const keys = exports.cache.keys();
    const keysToDelete = keys.filter(key => {
        if (keyPattern instanceof RegExp) {
            return keyPattern.test(key);
        }
        return key.includes(keyPattern);
    });
    exports.cache.del(keysToDelete);
}
//# sourceMappingURL=cache.js.map