import { RGBA, Paint } from "@figma/rest-api-spec";
export type CSSRGBAColor = `rgba(${number}, ${number}, ${number}, ${number})`;
export type CSSHexColor = `#${string}`;
export interface ColorValue {
    hex: CSSHexColor;
    opacity: number;
}
/**
 * Parameters for downloading an image from a Figma node
 */
export type FetchImageParams = {
    /**
     * The Node in Figma that will be rendered
     */
    nodeId: string;
    /**
     * The local file name to save the image
     */
    fileName: string;
    /**
     * The file mimetype for the image
     */
    fileType?: "png" | "svg";
};
/**
 * Parameters for downloading an image fill
 */
export type FetchImageFillParams = Omit<FetchImageParams, "fileType"> & {
    /**
     * Required to grab the background image when an image is used as a fill
     */
    imageRef: string;
};
/**
 * A simplified representation of a Figma paint
 */
export type SimplifiedFill = {
    type?: string;
    hex?: string;
    rgba?: string;
    opacity?: number;
    imageRef?: string;
    scaleMode?: string;
    gradientHandlePositions?: any[];
    gradientStops?: {
        position: number;
        color: ColorValue | string;
    }[];
} | CSSRGBAColor | CSSHexColor;
/**
 * A branded type for style IDs
 */
export type StyleId = `${string}_${string}` & {
    __brand: "StyleId";
};
/**
 * Format RGBA color into CSS RGBA string
 */
export declare function formatRGBAColor(color: RGBA, opacity?: number): CSSRGBAColor;
/**
 * Generates a CSS shorthand string from individual values
 * E.g. for padding or margin properties
 */
export declare function generateCSSShorthand(values: {
    top: number;
    right: number;
    bottom: number;
    left: number;
}): string;
/**
 * Check if a paint element is visible
 */
export declare function isVisible(paint: Paint | any): boolean;
/**
 * Parse a Figma paint object into a simplified fill representation
 */
export declare function parsePaint(paint: Paint): SimplifiedFill;
/**
 * Generate a unique variable ID with a given prefix
 */
export declare function generateVarId(prefix: string): StyleId;
/**
 * Remove keys with null or undefined values from an object
 */
export declare function removeEmptyKeys<T extends Record<string, any>>(obj: T): T;
/**
 * Download Figma image and save it locally
 * @param fileName - The filename to save as
 * @param localPath - The local path to save to
 * @param imageUrl - Image URL (images[nodeId])
 * @returns A Promise that resolves to the full file path where the image was saved
 * @throws Error if download fails
 */
export declare function downloadFigmaImage(fileName: string, localPath: string, imageUrl: string): Promise<string>;
