"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatRGBAColor = formatRGBAColor;
exports.generateCSSShorthand = generateCSSShorthand;
exports.isVisible = isVisible;
exports.parsePaint = parsePaint;
exports.generateVarId = generateVarId;
exports.removeEmptyKeys = removeEmptyKeys;
exports.downloadFigmaImage = downloadFigmaImage;
const axios_1 = __importDefault(require("axios"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
/**
 * Format RGBA color into CSS RGBA string
 */
function formatRGBAColor(color, opacity = 1) {
    const r = Math.round(color.r * 255);
    const g = Math.round(color.g * 255);
    const b = Math.round(color.b * 255);
    const a = color.a !== undefined ? color.a * opacity : opacity;
    return `rgba(${r}, ${g}, ${b}, ${a})`;
}
/**
 * Generates a CSS shorthand string from individual values
 * E.g. for padding or margin properties
 */
function generateCSSShorthand(values) {
    const { top, right, bottom, left } = values;
    // All sides are equal
    if (top === right && right === bottom && bottom === left) {
        return `${top}px`;
    }
    // Vertical sides are equal, horizontal sides are equal
    if (top === bottom && right === left) {
        return `${top}px ${right}px`;
    }
    // All sides are different
    return `${top}px ${right}px ${bottom}px ${left}px`;
}
/**
 * Check if a paint element is visible
 */
function isVisible(paint) {
    return paint.visible !== false && paint.opacity !== 0;
}
/**
 * Parse a Figma paint object into a simplified fill representation
 */
function parsePaint(paint) {
    // Handle solid colors
    if (paint.type === "SOLID" && paint.color) {
        return formatRGBAColor(paint.color, paint.opacity ?? 1);
    }
    // Handle image fills
    if (paint.type === "IMAGE" && paint.imageRef) {
        return {
            type: "IMAGE",
            imageRef: paint.imageRef,
            scaleMode: paint.scaleMode,
            opacity: paint.opacity
        };
    }
    // Handle gradients
    if ((paint.type === "GRADIENT_LINEAR" ||
        paint.type === "GRADIENT_RADIAL" ||
        paint.type === "GRADIENT_ANGULAR") &&
        paint.gradientStops) {
        return {
            type: paint.type,
            gradientHandlePositions: paint.gradientHandlePositions,
            gradientStops: paint.gradientStops.map(stop => ({
                position: stop.position,
                color: formatRGBAColor(stop.color, stop.color.a ?? 1)
            })),
            opacity: paint.opacity
        };
    }
    // Default fallback for other paint types
    return {
        type: paint.type,
        opacity: paint.opacity
    };
}
/**
 * Generate a unique variable ID with a given prefix
 */
function generateVarId(prefix) {
    const randomId = Math.random().toString(36).substring(2, 10);
    return `${prefix}_${randomId}`;
}
/**
 * Remove keys with null or undefined values from an object
 */
function removeEmptyKeys(obj) {
    return Object.fromEntries(Object.entries(obj).filter(([_, value]) => value !== null && value !== undefined));
}
/**
 * Download Figma image and save it locally
 * @param fileName - The filename to save as
 * @param localPath - The local path to save to
 * @param imageUrl - Image URL (images[nodeId])
 * @returns A Promise that resolves to the full file path where the image was saved
 * @throws Error if download fails
 */
async function downloadFigmaImage(fileName, localPath, imageUrl) {
    try {
        // Ensure local path exists
        if (!fs_1.default.existsSync(localPath)) {
            fs_1.default.mkdirSync(localPath, { recursive: true });
        }
        // Build the complete file path
        const fullPath = path_1.default.join(localPath, fileName);
        // Use axios to download the image, set responseType to stream
        const response = await (0, axios_1.default)({
            method: "GET",
            url: imageUrl,
            responseType: "stream",
            timeout: 30000, // 30 seconds timeout
        });
        // Create write stream
        const writer = fs_1.default.createWriteStream(fullPath);
        // Pipe response stream to file stream
        response.data.pipe(writer);
        // Return a Promise that resolves when writing is complete
        return new Promise((resolve, reject) => {
            writer.on("finish", () => {
                resolve(fullPath);
            });
            writer.on("error", (err) => {
                // Delete partially downloaded file
                fs_1.default.unlink(fullPath, () => { });
                reject(new Error(`Failed to download image: ${err.message}`));
            });
        });
    }
    catch (error) {
        const errorMessage = error instanceof Error ? error.message : String(error);
        throw new Error(`Error downloading image: ${errorMessage}`);
    }
}
//# sourceMappingURL=common.js.map