import { Rectangle, RGBA, Paint, StrokeWeights, HasLayoutTrait, HasFramePropertiesTrait } from '@figma/rest-api-spec';
/**
 * Type Definitions for Simplified Responses
 */
export type CSSHexColor = `#${string}`;
export type CSSRGBAColor = `rgba(${number}, ${number}, ${number}, ${number})`;
export type StyleId = `${string}_${string}` & {
    __brand: "StyleId";
};
export interface ColorValue {
    hex: CSSHexColor;
    opacity: number;
}
export type SimplifiedFill = {
    type?: string;
    hex?: string;
    rgba?: string;
    opacity?: number;
    imageRef?: string;
    scaleMode?: string;
    gradientHandlePositions?: any[];
    gradientStops?: {
        position: number;
        color: ColorValue | string;
    }[];
} | CSSRGBAColor | CSSHexColor;
/**
 * Parameters for downloading an image from a Figma node
 */
export type FetchImageParams = {
    /**
     * The Node in Figma that will be rendered
     */
    nodeId: string;
    /**
     * The local file name to save the image
     */
    fileName: string;
    /**
     * The file mimetype for the image
     */
    fileType?: "png" | "svg";
};
/**
 * Parameters for downloading an image fill
 */
export type FetchImageFillParams = Omit<FetchImageParams, "fileType"> & {
    /**
     * Required to grab the background image when an image is used as a fill
     */
    imageRef: string;
};
/**
 * Bounding box of a Figma node
 */
export interface BoundingBox {
    x: number;
    y: number;
    width: number;
    height: number;
}
/**
 * Simplified text styling
 */
export type TextStyle = Partial<{
    fontFamily: string;
    fontWeight: number;
    fontSize: number;
    lineHeight: string;
    letterSpacing: string;
    textCase: string;
    textAlignHorizontal: string;
    textAlignVertical: string;
}>;
/**
 * Simplified effects (shadows, blurs)
 */
export interface SimplifiedEffects {
    shadows?: {
        type: string;
        color: string;
        offset: {
            x: number;
            y: number;
        };
        radius: number;
        spread?: number;
    }[];
    blurs?: {
        type: string;
        radius: number;
    }[];
}
/**
 * Simplified stroke information
 */
export interface SimplifiedStroke {
    colors: string[];
    weight: number | {
        top: number;
        right: number;
        bottom: number;
        left: number;
    };
    align: string;
    dashPattern?: number[];
}
/**
 * Simplified layout information
 */
export interface SimplifiedLayout {
    direction?: string;
    spacing?: number;
    padding?: {
        top: number;
        right: number;
        bottom: number;
        left: number;
    };
    width?: number | string;
    height?: number | string;
    position?: string;
    alignment?: string;
}
/**
 * Types of style values
 */
export type StyleTypes = TextStyle | SimplifiedFill[] | SimplifiedLayout | SimplifiedStroke | SimplifiedEffects | string;
/**
 * Global variables container
 */
export interface GlobalVars {
    styles: Record<string, StyleTypes>;
}
/**
 * Simplified node representation
 */
export interface SimplifiedNode {
    id: string;
    name: string;
    type: string;
    boundingBox?: BoundingBox;
    text?: string;
    textStyle?: string;
    fills?: string;
    styles?: string;
    strokes?: string;
    effects?: string;
    opacity?: number;
    borderRadius?: string;
    layout?: string;
    children?: SimplifiedNode[];
}
/**
 * Simplified design representation
 */
export interface SimplifiedDesign {
    name: string;
    lastModified: string;
    thumbnailUrl: string;
    nodes: SimplifiedNode[];
    globalVars: GlobalVars;
}
/**
 * Type Guards
 */
/**
 * Check if a value is truthy (not null, undefined, false, 0, "", etc.)
 */
export declare function isTruthy<T>(val: T | null | undefined | false | 0 | ''): val is T;
/**
 * Check if an object has a property with a specific type
 * @param key - The property key to check
 * @param obj - The object to check
 * @param typeGuard - Optional function to check the type of the value
 * @returns Whether the object has the property with the right type
 */
export declare function hasValue<K extends PropertyKey, T>(key: K, obj: unknown, typeGuard?: (val: unknown) => val is T): obj is Record<K, T>;
/**
 * Check if an object is a Figma frame
 * @param val - The value to check
 * @returns Whether the value is a Figma frame
 */
export declare function isFrame(val: unknown): val is HasFramePropertiesTrait;
/**
 * Check if an object has layout traits
 * @param val - The value to check
 * @returns Whether the value has layout traits
 */
export declare function isLayout(val: unknown): val is HasLayoutTrait;
/**
 * Check if an object has stroke weights
 * @param val - The value to check
 * @returns Whether the value has stroke weights
 */
export declare function isStrokeWeights(val: unknown): val is StrokeWeights;
/**
 * Check if an object has a rectangle property
 * @param key - The property key to check
 * @param obj - The object to check
 * @returns Whether the object has a rectangle property
 */
export declare function isRectangle<T, K extends string>(key: K, obj: T): obj is T & {
    [P in K]: Rectangle;
};
/**
 * Check if a value is a rectangle corner radii array
 * @param val - The value to check
 * @returns Whether the value is a rectangle corner radii array
 */
export declare function isRectangleCornerRadii(val: unknown): val is number[];
/**
 * Check if a value is a CSS color value
 * @param val - The value to check
 * @returns Whether the value is a CSS color value
 */
export declare function isCSSColorValue(val: unknown): val is CSSRGBAColor | CSSHexColor;
/**
 * Color Utilities
 */
/**
 * Convert hex color value and opacity to rgba format
 * @param hex - Hexadecimal color value (e.g., "#FF0000" or "#F00")
 * @param opacity - Opacity value (0-1)
 * @returns Color string in rgba format
 */
export declare function hexToRgba(hex: string, opacity?: number): string;
/**
 * Convert color from RGBA to { hex, opacity }
 * @param color - The color to convert, including alpha channel
 * @param opacity - The opacity of the color, if not included in alpha channel
 * @returns The converted color
 */
export declare function convertColor(color: RGBA, opacity?: number): ColorValue;
/**
 * Convert color from Figma RGBA to rgba(#, #, #, #) CSS format
 * @param color - The color to convert, including alpha channel
 * @param opacity - The opacity of the color, if not included in alpha channel
 * @returns The converted color
 */
export declare function formatRGBAColor(color: RGBA, opacity?: number): CSSRGBAColor;
/**
 * Figma Utilities
 */
/**
 * Download Figma image and save it locally
 * @param fileName - The filename to save as
 * @param localPath - The local path to save to
 * @param imageUrl - Image URL (images[nodeId])
 * @returns A Promise that resolves to the full file path where the image was saved
 * @throws Error if download fails
 */
export declare function downloadFigmaImage(fileName: string, localPath: string, imageUrl: string): Promise<string>;
/**
 * Convert a Figma paint (solid, image, gradient) to a SimplifiedFill
 * @param raw - The Figma paint to convert
 * @returns The converted SimplifiedFill
 */
export declare function parsePaint(raw: Paint): SimplifiedFill;
/**
 * Check if an element is visible
 * @param element - The item to check
 * @returns True if the item is visible, false otherwise
 */
export declare function isVisible(element: {
    visible?: boolean;
}): boolean;
/**
 * General Utilities
 */
/**
 * Remove keys with empty arrays or empty objects from an object.
 * @param input - The input object or value.
 * @returns The processed object or the original value.
 */
export declare function removeEmptyKeys<T>(input: T): T;
/**
 * Generate a random variable ID
 * @param prefix - ID prefix
 * @returns A random ID string with prefix
 */
export declare function generateVarId(prefix?: string): StyleId;
/**
 * Generate a CSS shorthand for values that come with top, right, bottom, and left
 * @param values - The values to generate the shorthand for
 * @param options - Options for generating the shorthand
 * @returns The generated shorthand
 */
export declare function generateCSSShorthand(values: {
    top: number;
    right: number;
    bottom: number;
    left: number;
}, { ignoreZero, suffix, }?: {
    ignoreZero?: boolean;
    suffix?: string;
}): string | undefined;
