"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTruthy = isTruthy;
exports.hasValue = hasValue;
exports.isFrame = isFrame;
exports.isLayout = isLayout;
exports.isStrokeWeights = isStrokeWeights;
exports.isRectangle = isRectangle;
exports.isRectangleCornerRadii = isRectangleCornerRadii;
exports.isCSSColorValue = isCSSColorValue;
exports.hexToRgba = hexToRgba;
exports.convertColor = convertColor;
exports.formatRGBAColor = formatRGBAColor;
exports.downloadFigmaImage = downloadFigmaImage;
exports.parsePaint = parsePaint;
exports.isVisible = isVisible;
exports.removeEmptyKeys = removeEmptyKeys;
exports.generateVarId = generateVarId;
exports.generateCSSShorthand = generateCSSShorthand;
const axios_1 = __importDefault(require("axios"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
/**
 * Type Guards
 */
/**
 * Check if a value is truthy (not null, undefined, false, 0, "", etc.)
 */
function isTruthy(val) {
    return !!val;
}
/**
 * Check if an object has a property with a specific type
 * @param key - The property key to check
 * @param obj - The object to check
 * @param typeGuard - Optional function to check the type of the value
 * @returns Whether the object has the property with the right type
 */
function hasValue(key, obj, typeGuard) {
    const isObject = typeof obj === "object" && obj !== null;
    if (!isObject || !(key in obj))
        return false;
    const val = obj[key];
    return typeGuard ? typeGuard(val) : val !== undefined;
}
/**
 * Check if an object is a Figma frame
 * @param val - The value to check
 * @returns Whether the value is a Figma frame
 */
function isFrame(val) {
    return (typeof val === "object" &&
        !!val &&
        "clipsContent" in val &&
        typeof val.clipsContent === "boolean");
}
/**
 * Check if an object has layout traits
 * @param val - The value to check
 * @returns Whether the value has layout traits
 */
function isLayout(val) {
    return (typeof val === "object" &&
        !!val &&
        "absoluteBoundingBox" in val &&
        typeof val.absoluteBoundingBox === "object" &&
        !!val.absoluteBoundingBox &&
        "x" in val.absoluteBoundingBox &&
        "y" in val.absoluteBoundingBox &&
        "width" in val.absoluteBoundingBox &&
        "height" in val.absoluteBoundingBox);
}
/**
 * Check if an object has stroke weights
 * @param val - The value to check
 * @returns Whether the value has stroke weights
 */
function isStrokeWeights(val) {
    return (typeof val === "object" &&
        val !== null &&
        "top" in val &&
        "right" in val &&
        "bottom" in val &&
        "left" in val);
}
/**
 * Check if an object has a rectangle property
 * @param key - The property key to check
 * @param obj - The object to check
 * @returns Whether the object has a rectangle property
 */
function isRectangle(key, obj) {
    const recordObj = obj;
    return (typeof obj === "object" &&
        !!obj &&
        key in recordObj &&
        typeof recordObj[key] === "object" &&
        !!recordObj[key] &&
        "x" in recordObj[key] &&
        "y" in recordObj[key] &&
        "width" in recordObj[key] &&
        "height" in recordObj[key]);
}
/**
 * Check if a value is a rectangle corner radii array
 * @param val - The value to check
 * @returns Whether the value is a rectangle corner radii array
 */
function isRectangleCornerRadii(val) {
    return Array.isArray(val) && val.length === 4 && val.every((v) => typeof v === "number");
}
/**
 * Check if a value is a CSS color value
 * @param val - The value to check
 * @returns Whether the value is a CSS color value
 */
function isCSSColorValue(val) {
    return typeof val === "string" && (val.startsWith("#") || val.startsWith("rgba"));
}
/**
 * Color Utilities
 */
/**
 * Convert hex color value and opacity to rgba format
 * @param hex - Hexadecimal color value (e.g., "#FF0000" or "#F00")
 * @param opacity - Opacity value (0-1)
 * @returns Color string in rgba format
 */
function hexToRgba(hex, opacity = 1) {
    // Remove possible # prefix
    hex = hex.replace("#", "");
    // Handle shorthand hex values (e.g., #FFF)
    if (hex.length === 3) {
        hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
    }
    // Convert hex to RGB values
    const r = parseInt(hex.substring(0, 2), 16);
    const g = parseInt(hex.substring(2, 4), 16);
    const b = parseInt(hex.substring(4, 6), 16);
    // Ensure opacity is in the 0-1 range
    const validOpacity = Math.min(Math.max(opacity, 0), 1);
    return `rgba(${r}, ${g}, ${b}, ${validOpacity})`;
}
/**
 * Convert color from RGBA to { hex, opacity }
 * @param color - The color to convert, including alpha channel
 * @param opacity - The opacity of the color, if not included in alpha channel
 * @returns The converted color
 */
function convertColor(color, opacity = 1) {
    const r = Math.round(color.r * 255);
    const g = Math.round(color.g * 255);
    const b = Math.round(color.b * 255);
    // Alpha channel defaults to 1. If opacity and alpha are both < 1, their effects are multiplicative
    const a = Math.round(opacity * color.a * 100) / 100;
    const hex = ("#" +
        ((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1).toUpperCase());
    return { hex, opacity: a };
}
/**
 * Convert color from Figma RGBA to rgba(#, #, #, #) CSS format
 * @param color - The color to convert, including alpha channel
 * @param opacity - The opacity of the color, if not included in alpha channel
 * @returns The converted color
 */
function formatRGBAColor(color, opacity = 1) {
    const r = Math.round(color.r * 255);
    const g = Math.round(color.g * 255);
    const b = Math.round(color.b * 255);
    // Alpha channel defaults to 1. If opacity and alpha are both < 1, their effects are multiplicative
    const a = Math.round(opacity * color.a * 100) / 100;
    return `rgba(${r}, ${g}, ${b}, ${a})`;
}
/**
 * Figma Utilities
 */
/**
 * Download Figma image and save it locally
 * @param fileName - The filename to save as
 * @param localPath - The local path to save to
 * @param imageUrl - Image URL (images[nodeId])
 * @returns A Promise that resolves to the full file path where the image was saved
 * @throws Error if download fails
 */
async function downloadFigmaImage(fileName, localPath, imageUrl) {
    try {
        // Ensure local path exists
        if (!fs_1.default.existsSync(localPath)) {
            fs_1.default.mkdirSync(localPath, { recursive: true });
        }
        // Build the complete file path
        const fullPath = path_1.default.join(localPath, fileName);
        // Use axios to download the image, set responseType to stream
        const response = await (0, axios_1.default)({
            method: "GET",
            url: imageUrl,
            responseType: "stream",
            timeout: 30000, // 30 seconds timeout
        });
        // Create write stream
        const writer = fs_1.default.createWriteStream(fullPath);
        // Pipe response stream to file stream
        response.data.pipe(writer);
        // Return a Promise that resolves when writing is complete
        return new Promise((resolve, reject) => {
            writer.on("finish", () => {
                resolve(fullPath);
            });
            writer.on("error", (err) => {
                // Delete partially downloaded file
                fs_1.default.unlink(fullPath, () => { });
                reject(new Error(`Failed to download image: ${err.message}`));
            });
        });
    }
    catch (error) {
        const errorMessage = error instanceof Error ? error.message : String(error);
        throw new Error(`Error downloading image: ${errorMessage}`);
    }
}
/**
 * Convert a Figma paint (solid, image, gradient) to a SimplifiedFill
 * @param raw - The Figma paint to convert
 * @returns The converted SimplifiedFill
 */
function parsePaint(raw) {
    if (raw.type === "IMAGE") {
        return {
            type: "IMAGE",
            imageRef: raw.imageRef,
            scaleMode: raw.scaleMode,
        };
    }
    else if (raw.type === "SOLID") {
        // treat as SOLID
        const { hex, opacity } = convertColor(raw.color, raw.opacity);
        if (opacity === 1) {
            return hex;
        }
        else {
            return formatRGBAColor(raw.color, opacity);
        }
    }
    else if (["GRADIENT_LINEAR", "GRADIENT_RADIAL", "GRADIENT_ANGULAR", "GRADIENT_DIAMOND"].includes(raw.type) &&
        raw.gradientStops) {
        // treat as GRADIENT_LINEAR
        return {
            type: raw.type,
            gradientHandlePositions: raw.gradientHandlePositions,
            gradientStops: raw.gradientStops.map(({ position, color }) => ({
                position,
                color: convertColor(color),
            })),
        };
    }
    else {
        return {
            type: raw.type
        };
    }
}
/**
 * Check if an element is visible
 * @param element - The item to check
 * @returns True if the item is visible, false otherwise
 */
function isVisible(element) {
    return element.visible ?? true;
}
/**
 * General Utilities
 */
/**
 * Remove keys with empty arrays or empty objects from an object.
 * @param input - The input object or value.
 * @returns The processed object or the original value.
 */
function removeEmptyKeys(input) {
    // If not an object type or null, return directly
    if (typeof input !== "object" || input === null) {
        return input;
    }
    // Handle array type
    if (Array.isArray(input)) {
        return input.map((item) => removeEmptyKeys(item));
    }
    // Handle object type
    const result = {};
    for (const key in input) {
        if (Object.prototype.hasOwnProperty.call(input, key)) {
            const value = input[key];
            // Recursively process nested objects
            const cleanedValue = removeEmptyKeys(value);
            // Skip empty arrays and empty objects
            if (cleanedValue !== undefined &&
                !(Array.isArray(cleanedValue) && cleanedValue.length === 0) &&
                !(typeof cleanedValue === "object" &&
                    cleanedValue !== null &&
                    Object.keys(cleanedValue).length === 0)) {
                result[key] = cleanedValue;
            }
        }
    }
    return result;
}
/**
 * Generate a random variable ID
 * @param prefix - ID prefix
 * @returns A random ID string with prefix
 */
function generateVarId(prefix = "var") {
    const chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    let result = "";
    for (let i = 0; i < 6; i++) {
        const randomIndex = Math.floor(Math.random() * chars.length);
        result += chars[randomIndex];
    }
    return `${prefix}_${result}`;
}
/**
 * Generate a CSS shorthand for values that come with top, right, bottom, and left
 * @param values - The values to generate the shorthand for
 * @param options - Options for generating the shorthand
 * @returns The generated shorthand
 */
function generateCSSShorthand(values, { ignoreZero = true, suffix = "px", } = {}) {
    const { top, right, bottom, left } = values;
    if (ignoreZero && top === 0 && right === 0 && bottom === 0 && left === 0) {
        return undefined;
    }
    if (top === right && right === bottom && bottom === left) {
        return `${top}${suffix}`;
    }
    if (right === left) {
        if (top === bottom) {
            return `${top}${suffix} ${right}${suffix}`;
        }
        return `${top}${suffix} ${right}${suffix} ${bottom}${suffix}`;
    }
    return `${top}${suffix} ${right}${suffix} ${bottom}${suffix} ${left}${suffix}`;
}
//# sourceMappingURL=figma-helpers.js.map