"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSimplifiedLayout = buildSimplifiedLayout;
const identity_1 = require("../../utils/identity");
const common_1 = require("../../utils/common");
// Type guards using Record<string, unknown> to avoid extension issues
function hasFrameProps(node) {
    return (0, identity_1.isFrame)(node);
}
function hasLayoutProps(node) {
    return (0, identity_1.isLayout)(node);
}
// Type-safe accessors
function getFrameProp(node, prop) {
    if (hasFrameProps(node) && prop in node) {
        return node[prop];
    }
    return undefined;
}
function getLayoutProp(node, prop) {
    if (hasLayoutProps(node) && prop in node) {
        return node[prop];
    }
    return undefined;
}
// Convert Figma's layout config into a more typical flex-like schema
function buildSimplifiedLayout(n, parent) {
    const frameValues = buildSimplifiedFrameValues(n);
    const layoutValues = buildSimplifiedLayoutValues(n, parent, frameValues.mode) || {};
    return { ...frameValues, ...layoutValues };
}
// For flex layouts, process alignment and sizing
function convertAlign(axisAlign, stretch) {
    if (!axisAlign)
        return undefined;
    switch (axisAlign) {
        case "MIN":
            return "flex-start";
        case "MAX":
            return "flex-end";
        case "CENTER":
            return "center";
        case "SPACE_BETWEEN":
            return "space-between";
        case "BASELINE":
            return "baseline";
        default:
            return undefined;
    }
}
function convertSelfAlign(align) {
    if (!align)
        return undefined;
    switch (align) {
        case "MIN":
            return "flex-start";
        case "MAX":
            return "flex-end";
        case "CENTER":
            return "center";
        case "STRETCH":
            return "stretch";
        default:
            return undefined;
    }
}
function convertSizing(s) {
    if (!s)
        return undefined;
    switch (s) {
        case "FIXED":
            return "fixed";
        case "FILL":
            return "fill";
        case "HUG":
            return "hug";
        default:
            return undefined;
    }
}
function getDirection(axis, mode) {
    if (axis === "primary") {
        return mode === "row" ? "horizontal" : "vertical";
    }
    else {
        return mode === "row" ? "vertical" : "horizontal";
    }
}
function buildSimplifiedFrameValues(n) {
    if (!hasFrameProps(n)) {
        return { mode: "none" };
    }
    const overflowScroll = [];
    // Direction: Default to NONE
    const layoutMode = getFrameProp(n, 'layoutMode');
    const mode = !layoutMode || layoutMode === "NONE"
        ? "none"
        : layoutMode === "HORIZONTAL"
            ? "row"
            : "column";
    // Overflow
    const overflowDirection = getFrameProp(n, 'overflowDirection');
    if (overflowDirection?.includes("HORIZONTAL"))
        overflowScroll.push("x");
    if (overflowDirection?.includes("VERTICAL"))
        overflowScroll.push("y");
    // Handle all 'none' mode frames early
    if (mode === "none") {
        return { mode, overflowScroll: overflowScroll.length ? overflowScroll : undefined };
    }
    // Now we can safely process flex properties
    const frameValues = { mode };
    // Primary axis in Figma = justify-content (horizontal direction in row, vertical in column)
    const primaryAxisAlignItems = getFrameProp(n, 'primaryAxisAlignItems');
    frameValues.justifyContent = convertAlign(primaryAxisAlignItems ?? "MIN", {
        children: getFrameProp(n, 'children') || [],
        axis: "primary",
        mode,
    });
    // Counter axis in Figma = align-items (vertical direction in row, horizontal in column)
    const counterAxisAlignItems = getFrameProp(n, 'counterAxisAlignItems');
    frameValues.alignItems = convertAlign(counterAxisAlignItems ?? "MIN", {
        children: getFrameProp(n, 'children') || [],
        axis: "counter",
        mode,
    });
    // Layout alignment (individual node within parent)
    if (hasLayoutProps(n)) {
        frameValues.alignSelf = convertSelfAlign(getLayoutProp(n, 'layoutAlign'));
    }
    // Handle auto-layout properties
    const layoutWrap = getFrameProp(n, 'layoutWrap');
    frameValues.wrap = layoutWrap === "WRAP" ? true : undefined;
    const itemSpacing = getFrameProp(n, 'itemSpacing');
    frameValues.gap = itemSpacing ? `${itemSpacing}px` : undefined;
    // Handle padding if present
    const paddingTop = getFrameProp(n, 'paddingTop');
    const paddingRight = getFrameProp(n, 'paddingRight');
    const paddingBottom = getFrameProp(n, 'paddingBottom');
    const paddingLeft = getFrameProp(n, 'paddingLeft');
    if (paddingTop || paddingBottom || paddingLeft || paddingRight) {
        const valuesObj = {
            top: paddingTop ?? 0,
            right: paddingRight ?? 0,
            bottom: paddingBottom ?? 0,
            left: paddingLeft ?? 0,
        };
        frameValues.padding = (0, common_1.generateCSSShorthand)(valuesObj);
    }
    return frameValues;
}
function buildSimplifiedLayoutValues(n, parent, mode) {
    const layoutValues = { mode };
    if (!hasLayoutProps(n)) {
        return layoutValues;
    }
    // Sizing strategy
    layoutValues.sizing = {
        horizontal: convertSizing(getLayoutProp(n, 'layoutSizingHorizontal')),
        vertical: convertSizing(getLayoutProp(n, 'layoutSizingVertical')),
    };
    // Position
    if (parent && hasFrameProps(parent)) {
        const parentLayoutMode = getFrameProp(parent, 'layoutMode');
        const nodeLayoutPositioning = getLayoutProp(n, 'layoutPositioning');
        if ((parentLayoutMode === "NONE" || nodeLayoutPositioning === "ABSOLUTE")) {
            if (nodeLayoutPositioning === "ABSOLUTE") {
                layoutValues.position = "absolute";
            }
            const nodeBoundingBox = getLayoutProp(n, 'absoluteBoundingBox');
            const parentBoundingBox = getFrameProp(parent, 'absoluteBoundingBox');
            if (nodeBoundingBox && parentBoundingBox) {
                layoutValues.locationRelativeToParent = {
                    x: nodeBoundingBox.x - (parentBoundingBox?.x ?? nodeBoundingBox.x),
                    y: nodeBoundingBox.y - (parentBoundingBox?.y ?? nodeBoundingBox.y),
                };
            }
        }
    }
    // Handle dimensions if node has bounds
    if ((0, identity_1.isRectangle)("absoluteBoundingBox", n) && parent && (0, identity_1.isRectangle)("absoluteBoundingBox", parent)) {
        const bounds = n.absoluteBoundingBox;
        const layoutGrow = getLayoutProp(n, 'layoutGrow');
        const layoutAlign = getLayoutProp(n, 'layoutAlign');
        const layoutSizingHorizontal = getLayoutProp(n, 'layoutSizingHorizontal');
        const layoutSizingVertical = getLayoutProp(n, 'layoutSizingVertical');
        const hasHorizSizing = hasLayoutProps(n) && !(mode === "column" || mode === "none");
        if (hasHorizSizing) {
            if (!layoutGrow && layoutSizingHorizontal == "FIXED")
                layoutValues.dimensions = { ...(layoutValues.dimensions || {}), width: bounds.width };
            if (layoutAlign !== "STRETCH" && layoutSizingVertical == "FIXED")
                layoutValues.dimensions = { ...(layoutValues.dimensions || {}), height: bounds.height };
        }
        else {
            if (layoutAlign !== "STRETCH" && layoutSizingHorizontal == "FIXED")
                layoutValues.dimensions = { ...(layoutValues.dimensions || {}), width: bounds.width };
            if (!layoutGrow && layoutSizingVertical == "FIXED")
                layoutValues.dimensions = { ...(layoutValues.dimensions || {}), height: bounds.height };
        }
        const preserveRatio = getLayoutProp(n, 'preserveRatio');
        if (preserveRatio) {
            const aspectRatio = bounds.width / bounds.height;
            layoutValues.dimensions = { ...(layoutValues.dimensions || {}), aspectRatio };
        }
    }
    return layoutValues;
}
//# sourceMappingURL=layout.js.map