"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseFigmaUrl = parseFigmaUrl;
const errors_1 = require("./errors");
/**
 * Regular expressions for matching Figma URLs
 */
const FIGMA_URL_PATTERNS = {
    // Matches URLs like: https://www.figma.com/file/AbCdEfGhIjKl/FileName
    fileUrl: /figma\.com\/file\/([a-zA-Z0-9]+)(?:\/([^?]+))?/,
    // Matches URLs like: https://www.figma.com/proto/AbCdEfGhIjKl/FileName?node-id=123%3A456
    protoUrl: /figma\.com\/proto\/([a-zA-Z0-9]+)(?:\/([^?]+))?(?:.*node-id=([^&]+))?/,
    // Matches URLs like: https://www.figma.com/design/AbCdEfGhIjKl/FileName?node-id=123%3A456
    // Updated to be more flexible with various URL formats
    designUrl: /figma\.com\/design\/([a-zA-Z0-9]+)(?:\/([^?#]+))?(?:.*?node-id=([^&\s]+))?/
};
/**
 * Parses a Figma URL or reference to extract information
 * @param url The Figma URL or text reference to parse
 * @returns Object containing the fileKey (or textReference for non-URLs) and optional nodeId
 */
function parseFigmaUrl(url) {
    console.log(`Parsing Figma URL: ${url}`);
    if (!url) {
        console.error('No Figma URL or reference provided');
        throw new errors_1.FigmaError('No Figma URL or reference provided', 400);
    }
    // Check if this is a URL or a text reference
    if (!url.includes('figma.com')) {
        // It's a text reference, not a URL
        console.log(`Treating as text reference: ${url}`);
        return {
            textReference: url,
            // Use a placeholder fileKey that will be recognizable
            fileKey: 'text-reference-' + encodeURIComponent(url).substring(0, 20)
        };
    }
    // It's a URL, process normally
    let match;
    let nodeId;
    // Try to match file URL pattern
    match = url.match(FIGMA_URL_PATTERNS.fileUrl);
    if (match && match[1]) {
        console.log(`Matched file URL pattern: fileKey=${match[1]}`);
        return { fileKey: match[1] };
    }
    // Try to match proto URL pattern
    match = url.match(FIGMA_URL_PATTERNS.protoUrl);
    if (match && match[1]) {
        nodeId = match[3] ? decodeURIComponent(match[3]) : undefined;
        console.log(`Matched proto URL pattern: fileKey=${match[1]}, nodeId=${nodeId}`);
        return { fileKey: match[1], nodeId };
    }
    // Try to match design URL pattern
    match = url.match(FIGMA_URL_PATTERNS.designUrl);
    if (match && match[1]) {
        nodeId = match[3] ? decodeURIComponent(match[3]) : undefined;
        console.log(`Matched design URL pattern: fileKey=${match[1]}, nodeId=${nodeId}`);
        return { fileKey: match[1], nodeId };
    }
    // If none of the patterns above matched, try a more generic approach
    const genericFileKeyMatch = url.match(/figma\.com\/(?:file|proto|design)\/([a-zA-Z0-9]+)/);
    const genericNodeIdMatch = url.match(/node-id=([^&\s]+)/);
    if (genericFileKeyMatch && genericFileKeyMatch[1]) {
        const fileKey = genericFileKeyMatch[1];
        nodeId = genericNodeIdMatch && genericNodeIdMatch[1] ? decodeURIComponent(genericNodeIdMatch[1]) : undefined;
        console.log(`Matched generic pattern: fileKey=${fileKey}, nodeId=${nodeId}`);
        return { fileKey, nodeId };
    }
    // If no patterns match, throw an error
    console.error(`Invalid Figma URL: ${url}`);
    throw new errors_1.FigmaError(`Invalid Figma URL: ${url}`, 400);
}
//# sourceMappingURL=url-parser.js.map